"""
Module Description.


"""
import logging
from pathlib import Path

from mako.template import Template

DIR = Path(__file__).parent.absolute()

logger = logging.getLogger(__name__)


def writeTemplate(filename: str,
                  outputpath: Path,
                  data: dict = {},
                  templatepath: Path = None,
                  templateroot: Path = DIR / 'templates') -> None:
    """
    [summary]

    [extended_summary]

    Args:
        filename (str): [description]
        data (dict, optional): [description]. Defaults to {}.
        outputpath (Path, optional): [description]. Defaults to None.
    """
    if templatepath is None:
        path = templateroot
    else:
        path = templateroot / templatepath

    template = str(path / f'{filename}.tmpl')
    mytemplate = Template(filename=template, output_encoding='utf-8')
    outputpath.mkdir(exist_ok=True, parents=True)
    (outputpath / filename).write_bytes(mytemplate.render(**data))



"""
Module Description.


"""
import logging
import shutil
from pathlib import Path

from .figlet import figletise
from .templater import writeTemplate

DIR = Path(__file__).parent.absolute()
logger = logging.getLogger(__name__)


def copyTemplates(projectname: str, cli: bool):
    """
    Copy templates into folder structure.

    Args:
        projectname (str): The project name.
        cli (bool, optional): True if this is intended as a cli application. Defaults to False.
    """
    # Set the path for the main code repo
    PROJECT_ROOT = Path.cwd() / projectname
    PROJECT_PATH = PROJECT_ROOT / 'src' / projectname.replace('-', '_')
    PROJECT_PATH.mkdir(exist_ok=True, parents=True)
    (PROJECT_ROOT / 'src' / '__init__.py').touch(exist_ok=True)
    for file in (DIR / 'code').iterdir():
        if file.stem == 'scripts':
            shutil.copytree(file, PROJECT_ROOT / file.stem)
        elif file.stem == 'test':
            shutil.copytree(file, PROJECT_ROOT / file.stem)
        elif file.stem == '_config':
            shutil.copytree(file, PROJECT_ROOT / 'src' / file.stem)
        elif file.stem == 'main.py':
            data = {'projectname': projectname, 'cli': cli}
            writeTemplate('main.py', PROJECT_ROOT / 'src', data=data, templatepath=DIR / 'code')
        else:
            if not file.is_dir():
                shutil.copy(file, PROJECT_PATH / file.name)


def createFiles(projectname: str, cli: bool, python_version: str):
    PROJECT_ROOT = Path.cwd() / projectname
    data = {
        'projectname': projectname,
        'python_version': python_version,
        'figleted': figletise(projectname),
        'cli': cli,
        'author': None,
        'author_email': None,
        'description': None,
        'hashes': '##'
    }

    project = [
        '.env',
        '.flake8',
        '.gitignore',
        '.gitlab-ci.yml',
        '.style.yapf',
        'README.md',
        'Pipfile',
        'VERSION',
        'pytest.ini',
    ]

    for tmpl in project:
        writeTemplate(tmpl, PROJECT_ROOT, data=data, templatepath='project')

    vscode = [
        'launch.json',
        'settings.json',
        'extensions.json',
    ]
    for tmpl in vscode:
        writeTemplate(tmpl, PROJECT_ROOT / '.vscode', data=data, templatepath='vscode')

    package = [
        'LICENSE',
        'setup.cfg',
        'pyproject.toml',
        'MANIFEST.in',
        'deploy.sh',
    ]
    for tmpl in package:
        writeTemplate(tmpl, PROJECT_ROOT, data=data, templatepath='package')



# -*- coding: utf-8 -*-
"""
Module Description.


"""
import logging
from logging import config
from pathlib import Path

import yaml

DIR = Path(__file__).parent.absolute()
LOGPATH = DIR.parent.parent / 'logs'


def removeExistingLogs():
    """
    Remove old logs, so one clean log per run.
    """
    try:
        Path(LOGPATH / 'debug.log').unlink(missing_ok=True)
        Path(LOGPATH / 'debug.log.1').unlink(missing_ok=True)
        Path(LOGPATH / 'main.log').unlink(missing_ok=True)
    except Exception as e:
        print(e)


def cycleLogRuns(days: int = 10):
    """
    Save the old logs by date, keep a specificed number of runs and delte the rest.
    - Delete logs older than days
    - Save last log as a new date/run log
    - create new log
    """
    try:
        Path(LOGPATH / 'debug.log').unlink(missing_ok=True)
        Path(LOGPATH / 'debug.log.1').unlink(missing_ok=True)
        Path(LOGPATH / 'main.log').unlink(missing_ok=True)
    except Exception as e:
        print(e)


def configureLogging(clearLogs: bool = False):
    """."""
    if clearLogs:
        removeExistingLogs()

    Path(LOGPATH).mkdir(exist_ok=True, parents=True)
    mainfilename = LOGPATH / 'main.log'
    debugfilename = LOGPATH / 'debug.log'

    with open(DIR / 'logging.yaml', 'r') as f:
        log_cfg = yaml.full_load(f)

    log_cfg['handlers']['file_handler']['filename'] = mainfilename
    log_cfg['handlers']['rotating_handler']['filename'] = debugfilename

    config.dictConfig(log_cfg)

    # Set ERROR level logging on verbose modules
    modules = ['botocore', 'urllib3', 'googleapiclient', 'numba']
    for module in modules:
        logging.getLogger(module).setLevel(logging.ERROR)



# -*- coding: utf-8 -*-
"""
Module Description.


"""
import configparser
from pathlib import Path

DIR = Path(__file__).parent.absolute()


def getVersionFromSetup():
    """Get version from setup.cfg file."""
    config = configparser.ConfigParser()
    config.read(DIR.parent.parent / 'setup.cfg')
    return config.get('metadata', 'version')


def getVersion():
    """Get version from VERSION file."""
    with open(DIR.parent.parent / 'VERSION', 'r') as f:
        VERSION = f.read().strip()
    return VERSION



from .logger import configureLogging
from .settings import loadSettings, saveSettings
from .version import getVersion, getVersionFromSetup

__all__ = ['configureLogging', 'loadSettings', 'saveSettings', 'getVersion', 'getVersionFromSetup']



import shutil
from pathlib import Path

# import pytest
from src.create_pyproj.createfile import copyTemplates, createFiles

DIR = Path(__file__).parent.absolute()

projectname = 'z-test-proj'
cli = False
python_version = '3.8'

TEST_DIR = DIR.parent / projectname
PROJ_SRC = TEST_DIR / 'src'
PROJ_DIR = PROJ_SRC / projectname.replace('-', '_')


def test_copy_templates():
    if TEST_DIR.exists():
        shutil.rmtree(TEST_DIR)
    assert not TEST_DIR.exists(), f"{TEST_DIR} should not exist"
    copyTemplates(projectname, cli)
    assert (PROJ_SRC / '_config' /
            'logging.yaml').exists(), f'logging.yaml not copied to {PROJ_DIR}'


def test_create_files():
    createFiles(projectname, cli, python_version)
    assert (TEST_DIR / '.gitlab-ci.yml').exists(), f'gitlab-ci.yml not copied to {PROJ_DIR}'
    assert (TEST_DIR / 'pytest.ini').exists(), f'pytest.ini not copied to {TEST_DIR}'



import configparser
from pathlib import Path

CFGPATH = Path(__file__).parent.parent.parent


def getVersionFromSetup():
    config = configparser.ConfigParser()
    config.read(CFGPATH / 'setup.cfg')
    return config.get('metadata', 'version')



"""
Module Description.


"""
import logging
import os
from pathlib import Path

CWD = Path.cwd()
logger = logging.getLogger(__name__)


def initialise(projectname: str, no_init: bool, no_install: bool) -> None:
    PROJDIR = CWD.absolute() / projectname
    os.chdir(PROJDIR)

    # Whether to install in Pipenv
    if not no_install:
        os.system('pipenv install -d')

    # Whether to init the git repo
    if not no_init:
        git_init = ['git init --initial-branch=main', 'git add -A', 'git commit -m "first commit"']
        for cmd in git_init:
            os.system(cmd)



# -*- coding: utf-8 -*-
"""
Module Description.


"""
from pathlib import Path

import yaml
from pydantic import BaseSettings as PyBaseSettings

DIR = Path(__file__).parent.parent
SETTINGS_PATH = Path(DIR).absolute()


def saveSettings(settings: dict,
                 settings_file_name: str = 'settings',
                 settings_path: Path = SETTINGS_PATH,
                 sort_keys: bool = False) -> None:
    settings_path.mkdir(exist_ok=True, parents=True)
    with open(settings_path / f'{settings_file_name}.yaml', 'w') as f:
        yaml.dump(settings, f, sort_keys=sort_keys)


def loadSettings(settings_file_name: str = 'settings', settings_path: Path = SETTINGS_PATH) -> dict:
    if (not (settings_path / f'{settings_file_name}.yaml').exists()):
        saveSettings({})
    with open(settings_path / f'{settings_file_name}.yaml', 'r') as f:
        settings = yaml.full_load(f)
    return settings


class BaseSettings(PyBaseSettings):
    """
    An extension of Pydantic with a custom yaml loader

    https://pydantic-docs.helpmanual.io/usage/settings/#adding-sources

    Args:
        PyBaseSettings (BaseSettings): Pydantic basesettings
    """
    class Config:
        env_file_encoding = 'utf-8'

        @classmethod
        def customise_sources(
            cls,
            init_settings,
            env_settings,
            file_secret_settings,
        ):
            return (
                init_settings,
                env_settings,
                loadSettings,
                file_secret_settings,
            )



"""
Module Description.

pre-commit after running semantic_release
"""
import configparser
import logging
import sys
from pathlib import Path
from typing import Union

logger = logging.getLogger(__name__)


def read_version(VERSIONPATH: Path) -> str:
    with open(VERSIONPATH / 'VERSION', 'r') as f:
        VERSION = f.read().strip()
    return VERSION


def write_version(version: str, VERSIONPATH: Path) -> None:
    with open(VERSIONPATH / 'VERSION', 'w') as f:
        f.write(version)


def read_config(VERSIONPATH: Path):
    config = configparser.ConfigParser()
    config.read(VERSIONPATH / 'setup.cfg')
    return config


def write_config(VERSIONPATH: Union[Path, str], config: configparser.ConfigParser):
    with open(VERSIONPATH / 'setup.cfg', 'w') as f:
        config.write(f)


def resolve_path():
    VERSIONPATH = Path.cwd()
    if not (VERSIONPATH / 'VERSION').exists():
        if not (VERSIONPATH.parent / 'VERSION').exists():
            raise FileNotFoundError(f'Could not find VERSION file in {VERSIONPATH}')
        VERSIONPATH = VERSIONPATH.parent
    return VERSIONPATH


def update_version_files(version: str, VERSIONPATH: Path = None):
    if not VERSIONPATH:
        VERSIONPATH = resolve_path()

    write_version(version, VERSIONPATH)

    # If there is a setup.cfg, update it
    has_setup = (VERSIONPATH / 'setup.cfg').exists()
    if has_setup:
        config = read_config(VERSIONPATH)
        config.set('metadata', 'version', version)
        write_config(VERSIONPATH, config)


if __name__ == '__main__':
    version = sys.argv[1]
    update_version_files(version)



"""
Module Description.


"""
import argparse
import logging
import textwrap

from ._config import configureLogging
from .createfile import copyTemplates, createFiles
from .figlet import figletise
from .initialise import initialise

configureLogging()
logger = logging.getLogger(__name__)


def create_pyproj(args: argparse.Namespace):
    logger.info(f'Making new project...{figletise(args.projectname)}')

    try:
        copyTemplates(args.projectname, args.cli)
        createFiles(args.projectname, args.cli, args.python_version)
        initialise(args.projectname, args.no_init, args.no_install)
        logger.info(f'Install complete!\ncd into {args.projectname} and get developing...')
    except FileExistsError:
        logger.error(
            'The destination project already exists, please remove or choose another name.')


def main():
    parser = argparse.ArgumentParser(prog='create-pyproj',
                                     formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=textwrap.dedent('''\
    Create a new python skeleton project.
    --------------------------------
    The project has a number of development tools
    and convenince functions to get you started quickly!

    usage:
    create-pyproj <projectname> [options]

    The project structure will be copied to a folder ./<projectname>, the modules installed with Pipenv and a git repo initiated.
    '''))
    parser.add_argument('projectname', help='The name of the project you want to start.')
    parser.add_argument('--python_version',
                        action='store',
                        type=str,
                        default="3.10",
                        choices=['3.8', '3.9', '3.10'],
                        help='Select the python version. Defaults to 3.10.')
    parser.add_argument('--cli',
                        action='store_const',
                        const=True,
                        default=False,
                        help='Select this option if this is intended to run on the command line.')
    parser.add_argument('--no-init',
                        action='store_const',
                        const=True,
                        default=False,
                        help='Select this option to not initialise a git repo.')
    parser.add_argument('--no-install',
                        action='store_const',
                        const=True,
                        default=False,
                        help='Select this option to not install the packages.')

    args = parser.parse_args()
    create_pyproj(args)


if __name__ == "__main__":
    main()



from pathlib import Path

DIR = Path(__file__).parent
ROOT = DIR.parent


def replaceTextBetween(text: str, delimA: str, delimB: str, replacement: str) -> str:
    leadingText = text.split(delimA)[0]
    trailingText = text.split(delimB)[1]

    return leadingText + delimA + replacement + delimB + trailingText


def getbadges(badges: str = '\n***** Badges *******'):
    with open(ROOT / 'README.md', 'r') as f:
        readme = f.read()

    badge_start = '\n<!-- Badges -->'
    badge_end = '\n<!-- End Badges -->'
    newreadme = replaceTextBetween(readme, badge_start, badge_end, badges)

    with open(ROOT / 'README.md', 'w') as f:
        readme = f.write(newreadme)
